package org.jastadd.ragconnect.tests;

import defaultOnlyRead.ast.MqttHandler;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Tag;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.concurrent.TimeUnit;

/**
 * Base class for tests ensuring running mqtt broker.
 *
 * @author rschoene - Initial contribution
 */
@Tag("mqtt")
public abstract class AbstractMqttTest {

  static boolean checkDone = false;
  static Boolean checkResult;

  @BeforeAll
  public static void checkMqttConnection() {
    if (!checkDone) {
      checkDone = true;
      try {
        checkResult = new MqttHandler()
            .dontSendWelcomeMessage()
            .setHost(TestUtils.getMqttHost())
            .waitUntilReady(2, TimeUnit.SECONDS);
      } catch (IOException e) {
        checkResult = false;
      }
    }
    if (!checkResult) {
      throw new IllegalStateException("Mqtt Broker not ready!");
    }
  }

  @Test
  public final void buildModel() {
    createModel();
  }

  @Tag("mqtt")
  @Test
  public final void testCommunicateSendInitialValue() throws IOException, InterruptedException {
    createModel();
    setupReceiverAndConnect(true);

    communicateSendInitialValue();
  }

  protected abstract void communicateSendInitialValue() throws InterruptedException;

  @Tag("mqtt")
  @Test
  public final void testCommunicateOnlyUpdatedValue() throws IOException, InterruptedException {
    createModel();
    setupReceiverAndConnect(false);

    communicateOnlyUpdatedValue();
  }

  protected abstract void communicateOnlyUpdatedValue() throws InterruptedException;


  protected abstract void createModel();

  /**
   * Begin with this snippet
   * <pre>
   *     model.ragconnectSetupMqttWaitUntilReady(2, TimeUnit.SECONDS);
   *
   *     handler = new MqttHandler().dontSendWelcomeMessage().setHost(TestUtils.getMqttHost());
   *     assertTrue(handler.waitUntilReady(2, TimeUnit.SECONDS));
   * </pre>
   *
   * And then add dependencies, initialise receiver, add connections to those receivers,
   * and finally call generated connect* methods on model elements.
   * @param writeCurrentValue
   */
  protected abstract void setupReceiverAndConnect(boolean writeCurrentValue) throws IOException;

  @AfterEach
  public void alwaysCloseConnections() {
    closeConnections();
  }

  /**
   * Write the following snippet (using your correct handler and model):
   * <pre>
   * if (handler != null) {
   *    handler.close();
   * }
   * if (model != null) {
   *   model.ragconnectCloseConnections();
   * }
   * </pre>
   */
  protected abstract void closeConnections();

}
